@@ -23,7 +23,7 @@ filechunkname <- function (basenm, ndigs,nodenum=NULL)
 # chunk written to file outname (plus suffix based on node number) in
 # the node's global space
 filesort <- function(cls,basenm,ndigs,colnum,
-      outname,nsamp=1000,header=FALSE,sep=" ",fread=FALSE) 
+      outname,nsamp=1000,header=FALSE,sep=" ",usefread=FALSE) 
 {
    clusterEvalQ(cls,library(partools))
    setclsinfo(cls)
@@ -31,12 +31,8 @@ filesort <- function(cls,basenm,ndigs,colnum,
       header=header,sep=sep,nsamp) 
    samp <- Reduce(c,samps)
    bds <- getbounds(samp,length(cls))
-   if (fread) {
-     clusterEvalQ(cls,library(data.table))
-     clusterEvalQ(cls,myfread <- fread)
-   } else {clusterEvalQ(cls,myfread <- read.table)}
    invisible(clusterApply(cls,bds,mysortedchunk,
-      basenm,ndigs,colnum,outname,header,sep))
+      basenm,ndigs,colnum,outname,header,sep,usefread))
 }
 
 getsample <- function(basenm,ndigs,colnum,
@@ -58,18 +54,20 @@ getbounds <- function(samp,numnodes) {
    bds
 }
 
-mysortedchunk <-
-      function(mybds,basenm,ndigs,colnum,outname,header,sep) {
+mysortedchunk <- function(mybds,basenm,ndigs,colnum,outname,
+                    header,sep,usefread) {
    pte <- getpte()
    me <- pte$myid
    ncls <- pte$ncls
    mylo <- mybds[1]
    myhi <- mybds[2]
+   if (usefread) {
+      requireNamespace('data.table')
+      myfread <- data.table::fread
+   } else myfread <- read.table
    for (i in 1:ncls) {
-      tmp <-
-         myfread(filechunkname(basenm,ndigs,i),header=header,sep=sep)
-      # tmp <- freadfilechunkname(basenm,ndigs,i),header=header,sep)
-      tmpcol <- tmp[,colnum]
+      tmp <- myfread(filechunkname(basenm,ndigs,i),header=header,sep=sep) 
+      tmpcol <- tmp[,colnum,drop=FALSE]
       if (me == 1) {
          tmp <- tmp[tmpcol <= myhi,] 
       } else if (me == ncls) {
@@ -216,28 +214,35 @@ filecat <- function (cls, basenm, header = FALSE)  {
 }
 
 # saves the distributed data frame/matrix d to a distributed file of the
-# specified basename; the suffix has ndigs digits, and the field #
+# specified basename; the suffix has ndigs digits, and the field 
 # separator will be sep; d must have column names
 filesave <- function(cls,dname,newbasename,ndigs,sep) {
+   # what will the new file be called at each node?
    tmp <- paste('"',newbasename,'",',ndigs,sep='')
    cmd <- paste('myfilename <- filechunkname(',tmp,')',sep='')
    clusterExport(cls,"cmd",envir=environment())
    clusterEvalQ(cls,eval(parse(text=cmd)))
+   # start building the write.table() call
    tmp <- paste(dname,'myfilename',sep=',')
-   cnames <- colnames(get(dname))
-   clusterExport(cls,'cnames',envir=environment())
-   cmd <- paste('write.table(',tmp,
+   # what will the column names be for the new files?
+   clusterEvalQ(cls,eval(parse(text=cmd)))
+   cncmd <- paste('colnames(',dname,')',sep='')
+   clusterExport(cls,"cncmd",envir=environment())
+   clusterEvalQ(cls,cnames <- eval(parse(text=cncmd)))[[1]]
+   # now finish pasting the write.table() command, and run it
+   writecmd <- paste('write.table(',tmp,
       ',row.names=FALSE,col.names=cnames,sep="',sep,'")',sep='')
-   clusterExport(cls,"cmd",envir=environment())
-   clusterEvalQ(cls,docmd(cmd))
+   clusterExport(cls,"writecmd",envir=environment())
+   clusterEvalQ(cls,eval(parse(text=writecmd)))
 }
 
 # reads in a distributed file with prefix fname, producing a distributed
 # data frame dname
-fileread <- function(cls,fname,dname,ndigs,header=FALSE,sep=" ",fread=FALSE) {
-   if (fread) {
-     clusterEvalQ(cls,library(data.table))
-     clusterEvalQ(cls,myfread <- fread)
+fileread <- function(cls,fname,dname,ndigs,
+               header=FALSE,sep=" ",usefread=FALSE) {
+   if (usefread) {
+     clusterEvalQ(cls,requireNamespace('data.table'))
+     clusterEvalQ(cls,myfread <- data.table::fread)
    } else {clusterEvalQ(cls,myfread <- read.table)}
    fnameq <- paste("'",fname,"'",sep="")
    tmp <- paste(fnameq,ndigs,sep=',')
@@ -245,7 +250,6 @@ fileread <- function(cls,fname,dname,ndigs,header=FALSE,sep=" ",fread=FALSE) {
    clusterExport(cls,"cmd",envir=environment())
    clusterEvalQ(cls,eval(parse(text=cmd)))
    tmp <- paste(dname,"<- myfread(mychunk,header=",header,",sep='")
-   # tmp <- paste(dname,"<- fread(mychunk,header=",header,",sep='")
    cmd <- paste(tmp,sep,"')",sep="")
    clusterExport(cls,"cmd",envir=environment())
    invisible(clusterEvalQ(cls,eval(parse(text=cmd))))
